@@ -1,5 +1,5 @@
 package Ryu::Source;
-$Ryu::Source::VERSION = '0.012';
+$Ryu::Source::VERSION = '0.013';
 use strict;
 use warnings;
 
@@ -44,7 +44,7 @@ This is a coderef which should return a new L<Future>-compatible instance.
 
 Example overrides might include:
 
- $Ryu::Source::FUTURE_FACTORY = sub { Mojo::Future->new->set_label(shift) };
+ $Ryu::Source::FUTURE_FACTORY = sub { Mojo::Future->new->set_label($_[1]) };
 
 =cut
 
@@ -725,13 +725,14 @@ sub distinct_until_changed {
 
 =head2 sort_by
 
-Emits items sorted by the given key.
+Emits items sorted by the given key. This is a stable sort function.
 
 The algorithm is taken from L<List::UtilsBy>.
 
 =cut
 
 sub sort_by {
+    use sort qw(stable);
     my ($self, $code) = @_;
     my $src = $self->chained(label => (caller 0)[3] =~ /::([^:]+)$/);
     my @items;
@@ -750,13 +751,14 @@ sub sort_by {
 
 =head2 nsort_by
 
-Emits items numerically sorted by the given key.
+Emits items numerically sorted by the given key. This is a stable sort function.
 
 See L</sort_by>.
 
 =cut
 
 sub nsort_by {
+    use sort qw(stable);
     my ($self, $code) = @_;
     my $src = $self->chained(label => (caller 0)[3] =~ /::([^:]+)$/);
     my @items;
@@ -773,6 +775,58 @@ sub nsort_by {
     }, $src);
 }
 
+=head2 rev_sort_by
+
+Emits items sorted by the given key. This is a stable sort function.
+
+The algorithm is taken from L<List::UtilsBy>.
+
+=cut
+
+sub rev_sort_by {
+    use sort qw(stable);
+    my ($self, $code) = @_;
+    my $src = $self->chained(label => (caller 0)[3] =~ /::([^:]+)$/);
+    my @items;
+    my @keys;
+    $self->completed->on_done(sub {
+        $src->emit($_) for @items[sort { $keys[$b] cmp $keys[$a] } 0 .. $#items];
+    })->on_ready(sub {
+        return if $src->is_ready;
+        shift->on_ready($src->completed);
+    });
+    $self->each_while_source(sub {
+        push @items, $_;
+        push @keys, $_->$code;
+    }, $src);
+}
+
+=head2 rev_nsort_by
+
+Emits items numerically sorted by the given key. This is a stable sort function.
+
+See L</sort_by>.
+
+=cut
+
+sub rev_nsort_by {
+    use sort qw(stable);
+    my ($self, $code) = @_;
+    my $src = $self->chained(label => (caller 0)[3] =~ /::([^:]+)$/);
+    my @items;
+    my @keys;
+    $self->completed->on_done(sub {
+        $src->emit($_) for @items[sort { $keys[$b] <=> $keys[$a] } 0 .. $#items];
+    })->on_ready(sub {
+        return if $src->is_ready;
+        shift->on_ready($src->completed);
+    });
+    $self->each_while_source(sub {
+        push @items, $_;
+        push @keys, $_->$code;
+    }, $src);
+}
+
 =head2 extract_all
 
 Expects a regular expression and emits hashrefs containing
@@ -783,6 +837,7 @@ The regular expression will be applied using the m//gc operator.
 Example:
 
  $src->extract_all(qr{/(?<component>[^/]+)})
+ # emits { component => '...' }, { component => '...' }
 
 =cut
 
@@ -794,9 +849,7 @@ sub extract_all {
         shift->on_ready($src->completed);
     });
     $self->each_while_source(sub {
-        while(/$pattern/gc) {
-            $src->emit(+{ %+ });
-        }
+        $src->emit(+{ %+ }) while m/$pattern/gc;
     }, $src);
 }
 
